@@ -77,6 +77,7 @@ from picard.ui.passworddialog import (
 from picard.ui.playertoolbar import Player
 from picard.ui.searchdialog.album import AlbumSearchDialog
 from picard.ui.searchdialog.track import TrackSearchDialog
+from picard.ui.statusindicator import DesktopStatusIndicator
 from picard.ui.tagsfromfilenames import TagsFromFileNamesDialog
 from picard.ui.util import (
     MultiDirsSelectDialog,
@@ -115,6 +116,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.ignore_selection_changes = False
         self.toolbar = None
         self.player = None
+        self.status_indicators = []
         player = Player(self)
         if player.available:
             self.player = player
@@ -160,9 +162,8 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         if not self.show_cover_art_action.isChecked():
             self.cover_art_box.hide()
 
-        self.logDialog = LogView(self)
-        self.historyDialog = HistoryView(self)
-        self.optionsDialog = None
+        self.log_dialog = LogView(self)
+        self.history_dialog = HistoryView(self)
 
         bottomLayout = QtWidgets.QHBoxLayout()
         bottomLayout.setContentsMargins(0, 0, 0, 0)
@@ -193,6 +194,8 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
                 self.metadata_box.remove_selected_tags()
             else:
                 self.remove()
+        elif event.matches(QtGui.QKeySequence.Find):
+            self.search_edit.setFocus(True)
         else:
             super().keyPressEvent(event)
 
@@ -203,6 +206,11 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
             self.auto_update_check()
         self.metadata_box.restore_state()
 
+    def showEvent(self, event):
+        super().showEvent(event)
+        if DesktopStatusIndicator:
+            self.register_status_indicator(DesktopStatusIndicator(self.windowHandle()))
+
     def closeEvent(self, event):
         if config.setting["quit_confirmation"] and not self.show_quit_confirmation():
             event.ignore()
@@ -213,6 +221,9 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.saveWindowState()
         event.accept()
 
+    def register_status_indicator(self, indicator):
+        self.status_indicators.append(indicator)
+
     def show_quit_confirmation(self):
         unsaved_files = sum(a.get_num_unsaved_files() for a in self.tagger.albums.values())
         QMessageBox = QtWidgets.QMessageBox
@@ -242,6 +253,8 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         config.persist["window_state"] = self.saveState()
         isMaximized = int(self.windowState()) & QtCore.Qt.WindowMaximized != 0
         self.save_geometry()
+        self.log_dialog.save_geometry()
+        self.history_dialog.save_geometry()
         config.persist["window_maximized"] = isMaximized
         config.persist["view_cover_art"] = self.show_cover_art_action.isChecked()
         config.persist["view_toolbar"] = self.show_toolbar_action.isChecked()
@@ -267,7 +280,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     def create_statusbar(self):
         """Creates a new status bar."""
         self.statusBar().showMessage(_("Ready"))
-        self.infostatus = InfoStatus(self)
+        infostatus = InfoStatus(self)
         self.listening_label = QtWidgets.QLabel()
         self.listening_label.setVisible(False)
         self.listening_label.setToolTip("<qt/>" + _(
@@ -275,20 +288,24 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
             "you \"Search\" or \"Open in Browser\" from Picard, clicking the "
             "\"Tagger\" button on the web page loads the release into Picard."
         ))
-        self.statusBar().addPermanentWidget(self.infostatus)
+        self.statusBar().addPermanentWidget(infostatus)
         self.statusBar().addPermanentWidget(self.listening_label)
         self.tagger.tagger_stats_changed.connect(self.update_statusbar_stats)
         self.tagger.listen_port_changed.connect(self.update_statusbar_listen_port)
+        self.register_status_indicator(infostatus)
         self.update_statusbar_stats()
 
     @throttle(100)
     def update_statusbar_stats(self):
         """Updates the status bar information."""
-        self.infostatus.setFiles(len(self.tagger.files))
-        self.infostatus.setAlbums(len(self.tagger.albums))
-        self.infostatus.setPendingFiles(File.num_pending_files)
-        ws = self.tagger.webservice
-        self.infostatus.setPendingRequests(ws.num_pending_web_requests)
+        total_files = len(self.tagger.files)
+        total_albums = len(self.tagger.albums)
+        pending_files = File.num_pending_files
+        pending_requests = self.tagger.webservice.num_pending_web_requests
+
+        for indicator in self.status_indicators:
+            indicator.update(files=total_files, albums=total_albums,
+                pending_files=pending_files, pending_requests=pending_requests)
 
     def update_statusbar_listen_port(self, listen_port):
         if listen_port:
@@ -392,10 +409,10 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.add_files_action.setShortcut(QtGui.QKeySequence.Open)
         self.add_files_action.triggered.connect(self.add_files)
 
-        self.add_directory_action = QtWidgets.QAction(icontheme.lookup('folder'), _("A&dd Folder..."), self)
+        self.add_directory_action = QtWidgets.QAction(icontheme.lookup('folder'), _("Add Fold&er..."), self)
         self.add_directory_action.setStatusTip(_("Add a folder to the tagger"))
         # TR: Keyboard shortcut for "Add Directory..."
-        self.add_directory_action.setShortcut(QtGui.QKeySequence(_("Ctrl+D")))
+        self.add_directory_action.setShortcut(QtGui.QKeySequence(_("Ctrl+E")))
         self.add_directory_action.triggered.connect(self.add_directory)
 
         if self.show_close_window:
@@ -423,6 +440,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
 
         self.remove_action = QtWidgets.QAction(icontheme.lookup('list-remove'), _("&Remove"), self)
         self.remove_action.setStatusTip(_("Remove selected files/albums"))
+        self.remove_action.setShortcut(QtGui.QKeySequence.Delete)
         self.remove_action.setEnabled(False)
         self.remove_action.triggered.connect(self.remove)
 
@@ -486,6 +504,14 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.analyze_action.setShortcut(QtGui.QKeySequence(_("Ctrl+Y")))
         self.analyze_action.triggered.connect(self.analyze)
 
+        self.generate_fingerprints_action = QtWidgets.QAction(icontheme.lookup('fingerprint'), _("&Generate AcoustID Fingerprints"), self)
+        self.generate_fingerprints_action.setIconText(_("Generate Fingerprints"))
+        self.generate_fingerprints_action.setStatusTip(_("Generate the AcoustID audio fingerprints for the selected files without doing a lookup"))
+        self.generate_fingerprints_action.setEnabled(False)
+        self.generate_fingerprints_action.setToolTip(_('Generate the AcoustID audio fingerprints for the selected files'))
+        self.generate_fingerprints_action.setShortcut(QtGui.QKeySequence(_("Ctrl+Alt+F")))
+        self.generate_fingerprints_action.triggered.connect(self.generate_fingerprints)
+
         self.cluster_action = QtWidgets.QAction(icontheme.lookup('picard-cluster'), _("Cl&uster"), self)
         self.cluster_action.setStatusTip(_("Cluster files into album clusters"))
         self.cluster_action.setEnabled(False)
@@ -538,7 +564,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.view_log_action = QtWidgets.QAction(_("View &Error/Debug Log"), self)
         self.view_log_action.triggered.connect(self.show_log)
         # TR: Keyboard shortcut for "View Error/Debug Log"
-        self.view_log_action.setShortcut(QtGui.QKeySequence(_("Ctrl+E")))
+        self.view_log_action.setShortcut(QtGui.QKeySequence(_("Ctrl+G")))
 
         self.view_history_action = QtWidgets.QAction(_("View Activity &History"), self)
         self.view_history_action.triggered.connect(self.show_history)
@@ -661,6 +687,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         menu.addAction(self.cluster_action)
         menu.addAction(self.browser_lookup_action)
         menu.addSeparator()
+        menu.addAction(self.generate_fingerprints_action)
         menu.addAction(self.tags_from_filenames_action)
         menu.addAction(self.open_collection_in_browser_action)
         self.menuBar().addSeparator()
@@ -888,31 +915,23 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.tagger.activeWindow().close()
 
     def show_about(self):
-        self.show_options("about")
+        return self.show_options("about")
 
     def show_options(self, page=None):
-        if not self.optionsDialog:
-            self.optionsDialog = OptionsDialog(page, self)
-            self.optionsDialog.finished.connect(self.on_options_closed)
-        self.optionsDialog.show()
-        self.optionsDialog.raise_()
-        self.optionsDialog.activateWindow()
-
-    def on_options_closed(self):
-        self.optionsDialog = None
+        return OptionsDialog.show_instance(page, self)
 
     def show_help(self):
         webbrowser2.goto('documentation')
 
     def show_log(self):
-        self.logDialog.show()
-        self.logDialog.raise_()
-        self.logDialog.activateWindow()
+        self.log_dialog.show()
+        self.log_dialog.raise_()
+        self.log_dialog.activateWindow()
 
     def show_history(self):
-        self.historyDialog.show()
-        self.historyDialog.raise_()
-        self.historyDialog.activateWindow()
+        self.history_dialog.show()
+        self.history_dialog.raise_()
+        self.history_dialog.activateWindow()
 
     def open_bug_report(self):
         webbrowser2.goto('troubleshooting')
@@ -932,12 +951,16 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.panel.remove(self.selected_objects)
 
     def analyze(self):
-        if not config.setting['fingerprinting_system']:
-            if self.show_analyze_settings_info():
-                self.show_options("fingerprinting")
-            if not config.setting['fingerprinting_system']:
-                return
-        return self.tagger.analyze(self.selected_objects)
+        def callback(fingerprinting_system):
+            if fingerprinting_system:
+                self.tagger.analyze(self.selected_objects)
+        self._ensure_fingerprinting_configured(callback)
+
+    def generate_fingerprints(self):
+        def callback(fingerprinting_system):
+            if fingerprinting_system:
+                self.tagger.generate_fingerprints(self.selected_objects)
+        self._ensure_fingerprinting_configured(callback)
 
     def _openUrl(self, url):
         return QtCore.QUrl.fromLocalFile(url)
@@ -956,7 +979,17 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         for folder in folders:
             QtGui.QDesktopServices.openUrl(self._openUrl(folder))
 
-    def show_analyze_settings_info(self):
+    def _ensure_fingerprinting_configured(self, callback):
+        def on_finished(result):
+            callback(config.setting['fingerprinting_system'])
+        if not config.setting['fingerprinting_system']:
+            if self._show_analyze_settings_info():
+                dialog = self.show_options("fingerprinting")
+                dialog.finished.connect(on_finished)
+        else:
+            callback(config.setting['fingerprinting_system'])
+
+    def _show_analyze_settings_info(self):
         ret = QtWidgets.QMessageBox.question(self,
             _("Configuration Required"),
             _("Audio fingerprinting is not yet configured. Would you like to configure it now?"),
@@ -1036,6 +1069,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.save_action.setEnabled(can_save)
         self.view_info_action.setEnabled(can_view_info)
         self.analyze_action.setEnabled(can_analyze)
+        self.generate_fingerprints_action.setEnabled(have_files)
         self.refresh_action.setEnabled(can_refresh)
         self.autotag_action.setEnabled(can_autotag)
         self.browser_lookup_action.setEnabled(can_browser_lookup)
