@@ -39,7 +39,6 @@ from picard.const import (
     PLUGINS_API,
     USER_PLUGIN_DIR,
 )
-from picard.const.sys import IS_WIN
 from picard.util import reconnect
 
 from picard.ui import HashableTreeWidgetItem
@@ -114,6 +113,9 @@ class PluginTreeWidgetItem(HashableTreeWidgetItem):
     @staticmethod
     def set_icon(button, stdicon):
         button.setIcon(button.style().standardIcon(getattr(QtWidgets.QStyle, stdicon)))
+        # Workaround for Qt sometimes not updating the icon.
+        # See https://tickets.metabrainz.org/browse/PICARD-1647
+        button.repaint()
 
     def show_install(self, button, mode):
         if mode == 'hide':
@@ -370,6 +372,8 @@ class PluginsOptionsPage(OptionsPage):
                     self.set_current_item(item, scroll=True)
 
     def _reload(self):
+        if self.deleted:
+            return
         self._remove_all()
         self._populate()
         self._restore_plugins_states()
@@ -524,7 +528,8 @@ class PluginsOptionsPage(OptionsPage):
 
         if item.is_installed:
             item.buttons['install'].mode('hide')
-            item.buttons['uninstall'].mode('show')
+            item.buttons['uninstall'].mode(
+                'show' if plugin.is_user_installed else 'hide')
             item.enable(enabled, greyout=False)
 
             def uninstall_processor():
@@ -634,11 +639,7 @@ class PluginsOptionsPage(OptionsPage):
 
     @staticmethod
     def open_plugin_dir():
-        if IS_WIN:
-            url = 'file:///' + USER_PLUGIN_DIR
-        else:
-            url = 'file://' + USER_PLUGIN_DIR
-        QtGui.QDesktopServices.openUrl(QtCore.QUrl(url, QtCore.QUrl.TolerantMode))
+        QtGui.QDesktopServices.openUrl(QtCore.QUrl.fromLocalFile(USER_PLUGIN_DIR))
 
     def mimeTypes(self):
         return ["text/uri-list"]
